@@ -1,12 +1,25 @@
-/*
- * Gnocatan: a fun game.
- * (C) 1999 the Free Software Foundation
+/* Gnocatan - Implementation of the excellent Settlers of Catan board game.
+ *   Go buy a copy.
  *
- * Author: Dave Cole.
+ * Copyright (C) 1999 the Free Software Foundation
+ * Copyright (C) 2003 Bas Wijnen <b.wijnen@phys.rug.nl>
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * Implementation of the excellent Settlers of Catan board game.  Go
- * buy a copy.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
@@ -177,7 +190,7 @@ static ChainPart node_chain[] = {
 
 /* Build ring of nodes and edges around the current hex
  */
-static gboolean build_network(Map *map, Hex *hex, void *closure)
+static gboolean build_network(Map *map, Hex *hex, UNUSED(void *closure))
 {
 	Hex *adjacent[6];
 	gint idx;
@@ -244,7 +257,7 @@ static ChainConnect node_connect[] = {
 };
 /* Connect the the ring of nodes and edges to each other
  */
-static gboolean connect_network(Map *map, Hex *hex, void *closure)
+static gboolean connect_network(Map *map, Hex *hex, UNUSED(void *closure))
 {
 	Hex *adjacent[6];
 	gint idx;
@@ -337,7 +350,7 @@ static void layout_chits(Map *map)
 
 /* Randomise a map.  We do this by shuffling all of the land hexes,
  * and randomly reassigning port types.  This is the procedure
- * described in the board game rules.
+ * described in the board game rules.  Gold tiles are not shuffled
  */
 void map_shuffle_terrain(Map *map)
 {
@@ -355,7 +368,7 @@ void map_shuffle_terrain(Map *map)
 	for (x = 0; x < map->x_size; x++) {
 		for (y = 0; y < map->y_size; y++) {
 			Hex *hex = map->grid[y][x];
-			if (hex == NULL)
+			if (hex == NULL || hex->terrain == GOLD_TERRAIN)
 				continue;
 			if (hex->terrain == SEA_TERRAIN) {
 				if (hex->resource == NO_RESOURCE)
@@ -377,7 +390,7 @@ void map_shuffle_terrain(Map *map)
 			gint num;
 			gint idx;
 
-			if (hex == NULL)
+			if (hex == NULL || hex->terrain == GOLD_TERRAIN)
 				continue;
 			if (hex->terrain == SEA_TERRAIN) {
 				if (hex->resource == NO_RESOURCE)
@@ -423,6 +436,11 @@ Hex *map_robber_hex(Map *map)
 	return map->robber_hex;
 }
 
+Hex *map_pirate_hex(Map *map)
+{
+	return map->pirate_hex;
+}
+
 void map_move_robber(Map *map, gint x, gint y)
 {
 	if (map->robber_hex != NULL)
@@ -432,6 +450,11 @@ void map_move_robber(Map *map, gint x, gint y)
 		map->robber_hex->robber = TRUE;
 }
 
+void map_move_pirate(Map *map, gint x, gint y)
+{
+	map->pirate_hex = map_hex(map, x, y);
+}
+
 /* Allocate a new map
  */
 Map *map_new()
@@ -459,6 +482,20 @@ static Hex *copy_hex(Map *map, Hex *hex)
 	return copy;
 }
 
+static gboolean set_nosetup_nodes (UNUSED(Map *map), Hex *hex, Map *copy)
+{
+	gint idx;
+	for (idx = 0; idx < numElem(hex->nodes); ++idx) {
+		Node *node = hex->nodes[idx];
+		/* only handle nodes which are owned by the hex, to
+		 * prevent doing every node three times */
+		if (hex->x != node->x || hex->y != node->y) continue;
+		map_node (copy, node->x, node->y, node->pos)->no_setup
+			= node->no_setup;
+	}
+	return FALSE;
+}
+
 /* Make a copy of an existing map
  */
 Map *map_copy(Map *map)
@@ -474,7 +511,14 @@ Map *map_copy(Map *map)
 			copy->grid[y][x] = copy_hex(copy, map->grid[y][x]);
 	map_traverse(copy, build_network, NULL);
 	map_traverse(copy, connect_network, NULL);
+	map_traverse(map, (HexFunc)set_nosetup_nodes, copy);
 	copy->robber_hex = map->robber_hex;
+	copy->pirate_hex = map->pirate_hex;
+	copy->shrink_left = map->shrink_left;
+	copy->shrink_right = map->shrink_right;
+	copy->has_moved_ship = map->has_moved_ship;
+	copy->have_bridges = map->have_bridges;
+	copy->has_pirate = map->has_pirate;
 	copy->shrink_left = map->shrink_left;
 	copy->shrink_right = map->shrink_right;
 	/* chits is not owned by the map, i.e. not allocated / freed
@@ -519,8 +563,13 @@ void map_format_line(Map *map, gchar *line, gint y)
 		case DESERT_TERRAIN:
 			*line++ = 'd';
 			break;
+		case GOLD_TERRAIN:
+			*line++ = 'g';
+			break;
 		case SEA_TERRAIN:
 			*line++ = 's';
+			if (hex == map->pirate_hex)
+				*line++ = 'R';
 			if (hex->resource == NO_RESOURCE)
 				break;
 			switch (hex->resource) {
@@ -544,6 +593,8 @@ void map_format_line(Map *map, gchar *line, gint y)
 				break;
 			case NO_RESOURCE:
 				break;
+			case GOLD_RESOURCE:
+				g_assert (0);
 			}
 			*line++ = hex->facing + '0';
 			break;
@@ -563,11 +614,7 @@ void map_parse_line(Map *map, gchar *line)
 	gint x = 0;
 
 	for (;;) {
-		Terrain terrain = SEA_TERRAIN;
-		Resource resource = NO_RESOURCE;
-		gint facing = 0;
-		gint chit_pos = -1;
-		Hex *hex = NULL;
+		Hex *hex;
 
 		switch (*line++) {
 		case '\0':
@@ -578,80 +625,99 @@ void map_parse_line(Map *map, gchar *line)
 			x++;
 			continue;
 		case ',':
-			continue;
-		case 's': /* sea */
-			terrain = SEA_TERRAIN;
-			switch (*line++) {
-			case 'b':
-				resource = BRICK_RESOURCE;
-				break;
-			case 'g':
-				resource = GRAIN_RESOURCE;
-				break;
-			case 'o':
-				resource = ORE_RESOURCE;
-				break;
-			case 'w':
-				resource = WOOL_RESOURCE;
-				break;
-			case 'l':
-				resource = LUMBER_RESOURCE;
-				break;
-			case '?':
-				resource = ANY_RESOURCE;
-				break;
-			default:
-				resource = NO_RESOURCE;
-				--line;
-				break;
-			}
-			facing = 0;
-			if (resource != NO_RESOURCE) {
-				if (isdigit(*line))
-					facing = *line++ - '0';
-			}
-			break;
-		case 't': /* tree */
-			terrain = FOREST_TERRAIN;
-			break;
-		case 'p':
-			terrain = PASTURE_TERRAIN;
-			break;
-		case 'f':
-			terrain = FIELD_TERRAIN;
-			break;
-		case 'h':
-			terrain = HILL_TERRAIN;
-			break;
-		case 'm':
-			terrain = MOUNTAIN_TERRAIN;
-			break;
-		case 'd':
-			terrain = DESERT_TERRAIN;
-			break;
-		default:
+		case ' ':
+		case '\t':
 			continue;
 		}
+		--line;
 
 		if (x >= MAP_SIZE || map->y >= MAP_SIZE)
 			continue;
 
-		/* Read the chit sequence number
-		 */
-		if (isdigit(*line)) {
-			chit_pos = 0;
-			while (isdigit(*line))
-				chit_pos = chit_pos * 10 + *line++ - '0';
-		}
-
 		hex = g_malloc0(sizeof(*hex));
 		hex->map = map;
 		hex->y = map->y;
 		hex->x = x;
-		hex->terrain = terrain;
-		hex->resource = resource;
-		hex->facing = facing;
-		hex->chit_pos = chit_pos;
+		hex->terrain = SEA_TERRAIN;
+		hex->resource = NO_RESOURCE;
+		hex->facing = 0;
+		hex->chit_pos = -1;
+
+		switch (*line++) {
+		case 's': /* sea */
+			hex->terrain = SEA_TERRAIN;
+			if (*line == 'R') {
+				++line;
+				map->pirate_hex = hex;
+				map->has_pirate = TRUE;
+			}
+			switch (*line++) {
+			case 'b':
+				hex->resource = BRICK_RESOURCE;
+				break;
+			case 'g':
+				hex->resource = GRAIN_RESOURCE;
+				break;
+			case 'o':
+				hex->resource = ORE_RESOURCE;
+				break;
+			case 'w':
+				hex->resource = WOOL_RESOURCE;
+				break;
+			case 'l':
+				hex->resource = LUMBER_RESOURCE;
+				break;
+			case 'm': /* mine */
+				hex->resource = GOLD_RESOURCE;
+				break;
+			case '?':
+				hex->resource = ANY_RESOURCE;
+				break;
+			default:
+				hex->resource = NO_RESOURCE;
+				--line;
+				break;
+			}
+			hex->facing = 0;
+			if (hex->resource != NO_RESOURCE) {
+				if (isdigit(*line))
+					hex->facing = *line++ - '0';
+			}
+			break;
+		case 't': /* tree */
+			hex->terrain = FOREST_TERRAIN;
+			break;
+		case 'p':
+			hex->terrain = PASTURE_TERRAIN;
+			break;
+		case 'f':
+			hex->terrain = FIELD_TERRAIN;
+			break;
+		case 'h':
+			hex->terrain = HILL_TERRAIN;
+			break;
+		case 'm':
+			hex->terrain = MOUNTAIN_TERRAIN;
+			break;
+		case 'd':
+			hex->terrain = DESERT_TERRAIN;
+			break;
+		case 'g':
+			hex->terrain = GOLD_TERRAIN;
+			break;
+		default:
+			g_free (hex);
+			continue;
+		}
+
+		/* Read the chit sequence number
+		 */
+		if (isdigit(*line)) {
+			hex->chit_pos = 0;
+			while (isdigit(*line))
+				hex->chit_pos = hex->chit_pos * 10
+					+ *line++ - '0';
+		}
 
 		map->grid[map->y][x] = hex;
 		if (x >= map->x_size)
@@ -697,7 +763,7 @@ void map_set_chits(Map *map, GArray *chits)
 
 /* Disconnect a hex from all nodes and edges that it does not "own"
  */
-static gboolean disconnect_hex(Map *map, Hex *hex, void *closure)
+static gboolean disconnect_hex(UNUSED(Map *map), Hex *hex, UNUSED(void *closure))
 {
 	gint idx;
 
@@ -716,7 +782,7 @@ static gboolean disconnect_hex(Map *map, Hex *hex, void *closure)
 
 /* Free a node and all of the hexes and nodes that it is connected to.
  */
-static gboolean free_hex(Map *map, Hex *hex, void *closure)
+static gboolean free_hex(UNUSED(Map *map), Hex *hex, UNUSED(void *closure))
 {
 	gint idx;
 
