@@ -1,12 +1,25 @@
-/*
- * Gnocatan: a fun game.
- * (C) 1999 the Free Software Foundation
+/* Gnocatan - Implementation of the excellent Settlers of Catan board game.
+ *   Go buy a copy.
  *
- * Author: Dave Cole.
+ * Copyright (C) 1999 the Free Software Foundation
+ * Copyright (C) 2003 Bas Wijnen <b.wijnen@phys.rug.nl>
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * Implementation of the excellent Settlers of Catan board game.  Go
- * buy a copy.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
+#include "config.h"
 #include <math.h>
 #include <ctype.h>
 #include <string.h>
@@ -17,10 +30,10 @@
 #include "map.h"
 
 /* Local function prototypes */
-gboolean node_has_edge_owned_by(Node *node, gint owner, BuildType type);
-gboolean is_road_valid(Edge *edge, gint owner);
-gboolean is_ship_valid(Edge *edge, gint owner);
-gboolean is_bridge_valid(Edge *edge, gint owner);
+gboolean node_has_edge_owned_by(const Node *node, gint owner, BuildType type);
+gboolean is_road_valid(const Edge *edge, gint owner);
+gboolean is_ship_valid(const Edge *edge, gint owner);
+gboolean is_bridge_valid(const Edge *edge, gint owner);
 
 
 /* This file is broken into a number of sections:
@@ -45,14 +58,14 @@ gboolean is_bridge_valid(Edge *edge, gint owner);
 
 /* Return whether or not an edge is adjacent to a node
  */
-gboolean is_edge_adjacent_to_node(Edge *edge, Node *node)
+gboolean is_edge_adjacent_to_node(const Edge *edge, const Node *node)
 {
 	return edge->nodes[0] == node || edge->nodes[1] == node;
 }
 
 /* Return whether or not an edge is on land or coast
  */
-gboolean is_edge_on_land(Edge *edge)
+gboolean is_edge_on_land(const Edge *edge)
 {
 	gint idx;
 
@@ -65,24 +78,34 @@ gboolean is_edge_on_land(Edge *edge)
 	return FALSE;
 }
 
-/* Return whether or not an edge is on sea or coast
+/* Return whether or not an edge is on sea or coast (used only for ships)
  */
-gboolean is_edge_on_sea(Edge *edge)
+gboolean is_edge_on_sea(const Edge *edge)
 {
 	gint idx;
 
+	/* If the pirate is currently next to the edge, then specific sea
+	 * actions should not be possible (building ships is the only
+	 * specific sea action). */
+	for (idx = 0; idx < numElem(edge->hexes); idx++) {
+		Hex *hex = edge->hexes[idx];
+		if (hex && edge->map->pirate_hex == hex)
+			return FALSE;
+	}
+	/* The pirate is not next to the edge, return true if there is sea */
 	for (idx = 0; idx < numElem(edge->hexes); idx++) {
 		Hex *hex = edge->hexes[idx];
 		if (hex != NULL && hex->terrain == SEA_TERRAIN)
 			return TRUE;
 	}
 
+	/* There is no sea */
 	return FALSE;
 }
 
 /* Return whether or not a node is on land
  */
-gboolean is_node_on_land(Node *node)
+gboolean is_node_on_land(const Node *node)
 {
 	gint idx;
 
@@ -98,7 +121,7 @@ gboolean is_node_on_land(Node *node)
 /* Check if a node has a adjacent road/ship/bridge owned by the
  * specified player
  */
-gboolean node_has_edge_owned_by(Node *node, gint owner, BuildType type)
+gboolean node_has_edge_owned_by(const Node *node, gint owner, BuildType type)
 {
 	gint idx;
 
@@ -113,21 +136,21 @@ gboolean node_has_edge_owned_by(Node *node, gint owner, BuildType type)
 
 /* Check if a node has a adjacent road owned by the specified player
  */
-gboolean node_has_road_owned_by(Node *node, gint owner)
+gboolean node_has_road_owned_by(const Node *node, gint owner)
 {
 	return node_has_edge_owned_by(node, owner, BUILD_ROAD);
 }
 
 /* Check if a node has a adjacent ship owned by the specified player
  */
-gboolean node_has_ship_owned_by(Node *node, gint owner)
+gboolean node_has_ship_owned_by(const Node *node, gint owner)
 {
 	return node_has_edge_owned_by(node, owner, BUILD_SHIP);
 }
 
 /* Check if a node has a adjacent bridge owned by the specified player
  */
-gboolean node_has_bridge_owned_by(Node *node, gint owner)
+gboolean node_has_bridge_owned_by(const Node *node, gint owner)
 {
 	return node_has_edge_owned_by(node, owner, BUILD_BRIDGE);
 }
@@ -137,25 +160,24 @@ gboolean node_has_bridge_owned_by(Node *node, gint owner)
  * on them.  There is an exception when bridges are being used - two
  * buildings may be on adjacent nodes if separated by water.
  */
-gboolean is_node_spacing_ok(Node *node)
+gboolean is_node_spacing_ok(const Node *node)
 {
 	gint idx;
 
 	for (idx = 0; idx < numElem(node->edges); idx++) {
 		Edge *edge = node->edges[idx];
+		gint idx2;
 
 		if (edge == NULL)
 			continue;
 		if (node->map->have_bridges && !is_edge_on_land(edge))
 			continue;
-		else {
-			if (edge->nodes[0] == node) {
-				if (edge->nodes[1]->type != BUILD_NONE)
-					return FALSE;
-			} else {
-				if (edge->nodes[0]->type != BUILD_NONE)
-					return FALSE;
-			}
+		else for (idx2 = 0; idx2 < numElem(edge->nodes); ++idx2) {
+			Node *scan = edge->nodes[idx2];
+			if (scan == node)
+				continue;
+			if (scan->type != BUILD_NONE)
+				return FALSE;
 		}
 	}
 
@@ -164,7 +186,7 @@ gboolean is_node_spacing_ok(Node *node)
 
 /* Check if the specified node is next to the hex with the robber
  */
-gboolean is_node_next_to_robber(Node *node)
+gboolean is_node_next_to_robber(const Node *node)
 {
 	gint idx;
 
@@ -177,7 +199,7 @@ gboolean is_node_next_to_robber(Node *node)
 
 /* Check if a road has been positioned properly
  */
-gboolean is_road_valid(Edge *edge, gint owner)
+gboolean is_road_valid(const Edge *edge, gint owner)
 {
 	gint idx;
 
@@ -207,7 +229,7 @@ gboolean is_road_valid(Edge *edge, gint owner)
 
 /* Check if a ship has been positioned properly
  */
-gboolean is_ship_valid(Edge *edge, gint owner)
+gboolean is_ship_valid(const Edge *edge, gint owner)
 {
 	gint idx;
 
@@ -236,7 +258,7 @@ gboolean is_ship_valid(Edge *edge, gint owner)
 
 /* Check if a bridge has been positioned properly
  */
-gboolean is_bridge_valid(Edge *edge, gint owner)
+gboolean is_bridge_valid(const Edge *edge, gint owner)
 {
 	gint idx;
 
@@ -277,7 +299,7 @@ gboolean is_bridge_valid(Edge *edge, gint owner)
  * restrictions.  The server will enfore correct placement at the end
  * of the setup phase.
  */
-gboolean can_road_be_setup(Edge *edge, gint owner)
+gboolean can_road_be_setup(const Edge *edge, UNUSED(gint owner))
 {
 	return edge->owner < 0
 		&& is_edge_on_land(edge);
@@ -296,7 +318,7 @@ gboolean can_road_be_setup(Edge *edge, gint owner)
  * restrictions.  The server will enfore correct placement at the end
  * of the setup phase.
  */
-gboolean can_ship_be_setup(Edge *edge, gint owner)
+gboolean can_ship_be_setup(const Edge *edge, UNUSED(gint owner))
 {
 	return edge->owner < 0
 		&& is_edge_on_sea(edge);
@@ -315,7 +337,7 @@ gboolean can_ship_be_setup(Edge *edge, gint owner)
  * restrictions.  The server will enfore correct placement at the end
  * of the setup phase.
  */
-gboolean can_bridge_be_setup(Edge *edge, gint owner)
+gboolean can_bridge_be_setup(const Edge *edge, UNUSED(gint owner))
 {
 	return edge->owner < 0
 		&& !is_edge_on_land(edge);
@@ -333,7 +355,7 @@ gboolean can_bridge_be_setup(Edge *edge, gint owner)
  *     owned by the specifed player, but not separated by a building
  *     owned by a different player.
  */
-gboolean can_road_be_built(Edge *edge, gint owner)
+gboolean can_road_be_built(const Edge *edge, gint owner)
 {
 	return edge->owner < 0
 		&& is_road_valid(edge, owner);
@@ -351,12 +373,56 @@ gboolean can_road_be_built(Edge *edge, gint owner)
  *     owned by the specifed player, but not separated by a building
  *     owned by a different player.
  */
-gboolean can_ship_be_built(Edge *edge, gint owner)
+gboolean can_ship_be_built(const Edge *edge, gint owner)
 {
 	return edge->owner < 0
 		&& is_ship_valid(edge, owner);
 }
 
+/* Helper function for can_ship_be_moved */
+static gboolean can_ship_be_moved_node (const Node *node, gint owner, const Edge *not)
+{
+	gint idx;
+	/* if a building of a different player is on it, it is
+	 * unconnected */
+	if (node->type != BUILD_NONE && node->owner != owner)
+		return TRUE;
+	/* if there is a building of the player, it is connected */
+	if (node->type != BUILD_NONE) return FALSE;
+	/* no buildings: check all edges for ships */
+	for (idx = 0; idx < numElem(node->edges); idx++) {
+		Edge *edge = node->edges[idx];
+		/* If this is a ship of the player, it is connected */
+		if (edge && edge->owner == owner && edge != not
+			&& edge->type == BUILD_SHIP) return FALSE;
+	}
+	return TRUE;
+}
+
+/* Edge cursor check function.
+ *
+ * Determine whether or not a ship can be moved from this edge by the
+ * specified player.  Perform the following checks:
+ *
+ * 1 - Edge must currently have a ship on it.
+ * 2 - On one side, there must be neither a building, nor a ship of the
+ *     specified player.  A ship is allowed, if there is building of a
+ *     different player in between.
+ */
+gboolean can_ship_be_moved(const Edge *edge, gint owner)
+{
+	gint idx;
+	/* edge must be a ship of the correct user */
+	if (edge->owner != owner || edge->type != BUILD_SHIP) return FALSE;
+	/* if the pirate is next to the edge, it is not allowed to move */
+	if (!is_edge_on_sea (edge) ) return FALSE;
+	/* check all nodes, until one is found that is not connected */
+	for (idx = 0; idx < numElem(edge->nodes); idx++)
+		if (can_ship_be_moved_node (edge->nodes[idx], owner, edge))
+			return TRUE;
+	return FALSE;
+}
+
 /* Edge cursor check function.
  *
  * Determine whether or not a bridge can be built on this edge by the
@@ -369,7 +435,7 @@ gboolean can_ship_be_built(Edge *edge, gint owner)
  *     segment owned by the specifed player, but not separated by a
  *     building owned by a different player.
  */
-gboolean can_bridge_be_built(Edge *edge, gint owner)
+gboolean can_bridge_be_built(const Edge *edge, gint owner)
 {
 	return edge->owner < 0
 		&& is_bridge_valid(edge, owner);
@@ -381,18 +447,20 @@ gboolean can_bridge_be_built(Edge *edge, gint owner)
  * the specified player during the setup phase.  Perform the following
  * checks:
  *
- * 1 - Node must be vacant.
- * 2 - Node must be adjacent to a land hex.
- * 3 - Node must not be within one node of another building.
+ * 1 - Node must not be in the no-setup list.
+ * 2 - Node must be vacant.
+ * 3 - Node must be adjacent to a land hex.
+ * 4 - Node must not be within one node of another building.
  *
  * The checks are not as strict as for normal play.  This allows the
  * player to try a few different configurations without layout
  * restrictions.  The server will enfore correct placement at the end
  * of the setup phase.
  */
-gboolean can_settlement_be_setup(Node *node, gint owner)
+gboolean can_settlement_be_setup(const Node *node, UNUSED(gint owner))
 {
-	return node->owner < 0
+	return !node->no_setup
+		&& node->owner < 0
 		&& is_node_on_land(node)
 		&& is_node_spacing_ok(node);
 }
@@ -407,7 +475,7 @@ gboolean can_settlement_be_setup(Node *node, gint owner)
  * 3 - Node must be adjacent to a land hex.
  * 4 - Node must not be within one node of another building.
  */
-gboolean can_settlement_be_built(Node *node, gint owner)
+gboolean can_settlement_be_built(const Node *node, gint owner)
 {
 	return node->owner < 0
 		&& (node_has_road_owned_by(node, owner)
@@ -422,7 +490,7 @@ gboolean can_settlement_be_built(Node *node, gint owner)
  * Determine whether or not a settlement can be upgraded to a city by
  * the specified player.
  */
-gboolean can_settlement_be_upgraded(Node *node, gint owner)
+gboolean can_settlement_be_upgraded(const Node *node, gint owner)
 {
 	return node->owner == owner
 		&& node->type == BUILD_SETTLEMENT;
@@ -441,7 +509,7 @@ gboolean can_settlement_be_upgraded(Node *node, gint owner)
  * 4 - If vacent, node must not be within one node of another
  *     building.
  */
-gboolean can_city_be_built(Node *node, gint owner)
+gboolean can_city_be_built(const Node *node, gint owner)
 {
 	if (can_settlement_be_upgraded(node, owner))
 		return TRUE;
@@ -454,21 +522,25 @@ gboolean can_city_be_built(Node *node, gint owner)
 		&& is_node_spacing_ok(node);
 }
 
-/* Node cursor check function.
+/* Hex cursor check function.
  *
  * Determine whether or not the robber be moved to the specified hex.
  *
  * Can only move the robber to hex which produces resources (roll >
  * 0).  We cannot move the robber to the same hex it is already on.
+ * Also check if pirate can be moved.
  */
-gboolean can_robber_be_moved(Hex *hex, gint owner)
+gboolean can_robber_or_pirate_be_moved(const Hex *hex, UNUSED(gint owner))
 {
-	return hex->roll > 0 && !hex->robber;
+	if (hex->terrain == SEA_TERRAIN)
+		return (hex->map->has_pirate) && (hex != hex->map->pirate_hex);
+	else
+		return (hex->roll > 0) && (!hex->robber);
 }
 
 /* Iterator function for map_can_place_road() query
  */
-static gboolean can_place_road_check(Map *map, Hex *hex, gint *owner)
+static gboolean can_place_road_check(UNUSED(Map *map), Hex *hex, gint *owner)
 {
 	gint idx;
 
@@ -480,7 +552,7 @@ static gboolean can_place_road_check(Map *map, Hex *hex, gint *owner)
 
 /* Iterator function for map_can_place_ship() query
  */
-static gboolean can_place_ship_check(Map *map, Hex *hex, gint *owner)
+static gboolean can_place_ship_check(UNUSED(Map *map), Hex *hex, gint *owner)
 {
 	gint idx;
 
@@ -492,7 +564,7 @@ static gboolean can_place_ship_check(Map *map, Hex *hex, gint *owner)
 
 /* Iterator function for map_can_place_bridge() query
  */
-static gboolean can_place_bridge_check(Map *map, Hex *hex, gint *owner)
+static gboolean can_place_bridge_check(UNUSED(Map *map), Hex *hex, gint *owner)
 {
 	gint idx;
 
@@ -534,7 +606,7 @@ gboolean map_can_place_bridge(Map *map, gint owner)
 
 /* Iterator function for map_can_place_settlement() query
  */
-static gboolean can_place_settlement_check(Map *map, Hex *hex, gint *owner)
+static gboolean can_place_settlement_check(UNUSED(Map *map), Hex *hex, gint *owner)
 {
 	gint idx;
 
@@ -556,7 +628,7 @@ gboolean map_can_place_settlement(Map *map, gint owner)
 
 /* Iterator function for map_can_upgrade_settlement() query
  */
-static gboolean can_upgrade_settlement_check(Map *map, Hex *hex, gint *owner)
+static gboolean can_upgrade_settlement_check(UNUSED(Map *map), Hex *hex, gint *owner)
 {
 	gint idx;
 
@@ -602,7 +674,7 @@ gboolean map_building_spacing_ok(Map *map, gint owner,
 
 /* Ignoring building spacing, check if the building connects to a road.
  */
-gboolean map_building_connect_ok(Map *map, gint owner, BuildType type,
+gboolean map_building_connect_ok(Map *map, gint owner, UNUSED(BuildType type),
 				 gint x, gint y, gint pos)
 {
 	Node *node = map_node(map, x, y, pos);
@@ -630,6 +702,7 @@ gboolean map_building_vacant(Map *map, BuildType type,
 			|| node->type == BUILD_SETTLEMENT;
 	case BUILD_ROAD:
 	case BUILD_SHIP:
+	case BUILD_MOVE_SHIP:
 	case BUILD_BRIDGE:
 		g_error("map_building_vacant() called with edge");
 		return FALSE;
@@ -694,232 +767,90 @@ gboolean map_bridge_connect_ok(Map *map, gint owner, gint x, gint y, gint pos)
 	return is_bridge_valid(edge, owner);
 }
 
-static gint calc_road_length(Edge *edge);
-
-/* Node has been visited via one edge, of the two remaining edges,
- * return the length of the longest unvisited path from the fork.
- */
-static gint calc_longest_fork(Node *node, gint owner)
-{
-	gint idx;
-	gint max_len;
-
-	/* Cannot go past node if it is occupied by another
-	 * player
-	 */
-	if (node->type != BUILD_NONE && node->owner != owner)
-		return 0;
-
-	node->visited = 2;
-
-	/* Try all roads adjacent to the node that we have not
-	 * visited yet
-	 */
-	max_len = 0;
-	for (idx = 0; idx < numElem(node->edges); idx++) {
-		Edge *edge = node->edges[idx];
-		gint len;
-
-		if (edge->owner != owner || edge->visited == 2)
-			continue;
-		len = calc_road_length(edge);
-		if (len > max_len)
-			max_len = len;
-	}
-
-	node->visited = 1;
-
-	return max_len;
+static BuildType bridge_as_road(BuildType type) {
+	if (type == BUILD_BRIDGE)
+		return BUILD_ROAD;
+	else
+		return type;
 }
 
-/* Return the length of unvisited road connected to this edge
- * (including this edge).  We must only count edges in one direction
- * to protect against cycles.
- */
-static gint calc_road_length(Edge *edge)
+/* calculate the longest road */
+static gint find_longest_road_recursive (Edge *edge)
 {
-	gint idx;
 	gint len = 0;
-
-	edge->visited = 2;
-
-	for (idx = 0; idx < numElem(edge->nodes); idx++) {
-		Node *node = edge->nodes[idx];
-
-		if (node->visited == 2)
-			continue;
-		len = calc_longest_fork(node, edge->owner);
-		if (len > 0)
-			break;
-	}
-
-	edge->visited = 1;
-
-	return 1 + len;
-}
-
-static GList *find_tail_edges(Edge *edge);
-
-static GList *node_find_tail_edges(Node *node, gint owner)
-{
-	gint idx;
-	GList *tails = NULL;
-
-	/* Cannot go past node if it is occupied by another
-	 * player
-	 */
-	if (node->type != BUILD_NONE && node->owner != owner)
-		return NULL;
-
-	node->visited = 1;
-	/* Try all roads adjacent to the node that we have not
-	 * visited yet
-	 */
-	for (idx = 0; idx < numElem(node->edges); idx++) {
-		Edge *edge = node->edges[idx];
-
-		if (edge->owner != owner || edge->visited)
-			continue;
-		tails = g_list_concat(tails, find_tail_edges(edge));
-	}
-
-	return tails;
-}
-
-static gboolean node_has_other_edges(Node *node, Edge *edge)
-{
-	gint idx;
-
-	for (idx = 0; idx < numElem(node->edges); idx++)
-		if (node->edges[idx] != edge
-		    && node->edges[idx]->owner == edge->owner)
-			return TRUE;
-	return FALSE;
-}
-
-/* Return a list of tail edges that are in the same group as this one
- */
-static GList *find_tail_edges(Edge *edge)
-{
-	GList *tails = NULL;
-	gint idx;
-	gint num_exits;
-
-	edge->visited = 1;
-
-	/* Count the number of exits from this edge
-	 */
-	num_exits = 0;
-	for (idx = 0; idx < numElem(edge->nodes); idx++) {
-		Node *node = edge->nodes[idx];
+	gint nodeidx, edgeidx;
+	edge->visited = TRUE;
+	/* check all nodes to see which one make the longer road. */
+	for (nodeidx = 0; nodeidx < numElem(edge->nodes); nodeidx++) {
+		Node *node = edge->nodes[nodeidx];
+		/* don't go back to where we came from */
+		if (node->visited) continue;
+		/* don't continue counting if someone else's building is on
+		 * the node. */
 		if (node->type != BUILD_NONE && node->owner != edge->owner)
 			continue;
-		if (node_has_other_edges(node, edge))
-			num_exits++;
+		/* don't let other go back here */
+		node->visited = TRUE;
+		/* try all edges */
+		for (edgeidx = 0; edgeidx < numElem(node->edges); edgeidx++) {
+			Edge *here = node->edges[edgeidx];
+			if (here && !here->visited
+				&& here->owner == edge->owner) {
+				/* don't allow ships to extend roads, except
+				 * if there is a construction in between */
+				/* bridges are treated as roads */
+				if (node->type != BUILD_NONE ||
+					bridge_as_road(here->type) == bridge_as_road(edge->type)) {
+					gint thislen = find_longest_road_recursive (here);
+					/* take the maximum of all paths */
+					if (thislen > len) len = thislen;
+				}
+			}
+		}
+		/* Allow other roads to use this node again. */
+		node->visited = FALSE;
 	}
-	if (num_exits < 2)
-		/* This edge is either a singleton, or a tail.
-		 */
-		tails = g_list_prepend(NULL, edge);
-
-	/* Now traverse to all edges that are accessible from this one
-	 * and collect their tail lists.
-	 */
-	for (idx = 0; idx < numElem(edge->nodes); idx++) {
-		Node *node = edge->nodes[idx];
-		if (node->visited)
-			continue;
-		tails = g_list_concat(tails,
-				      node_find_tail_edges(node, edge->owner));
-	}
-
-	return tails;
+	edge->visited = FALSE;
+	return len + 1;
 }
 
-/* For all roads the are "owned" by this hex, work out the length of
- * the road.  If the length of the road exceeds the previous maximum
- * for the owning player, update the maximum.
- */
-static gboolean find_longest_road(Map *map, Hex *hex, gint *lengths)
+static gboolean find_longest_road(UNUSED(Map *map), Hex *hex, gint *lengths)
 {
 	gint idx;
-
 	for (idx = 0; idx < numElem(hex->edges); idx++) {
 		Edge *edge = hex->edges[idx];
 		gint len;
-		GList *tails;
-
+		/* skip unowned edges, and edges that will be handled by
+		 * other hexes */
 		if (edge->owner < 0 || edge->x != hex->x || edge->y != hex->y)
 			continue;
-		if (edge->visited)
-			continue;
-
-		/* Build a list of all "tail" edges in connected edge group.
-		 */
-		tails = find_tail_edges(edge);
-		if (tails == NULL)
-			/* There are no tails, so the edge group must
-			 * be a cycle.  This means we can start
-			 * anywhere in the group.
-			 */
-			tails = g_list_append(NULL, edge);
-		/* Work out the longest road that can be traced from
-		 * each "tail" edge.
-		 */
-                while (tails != NULL) {
-			Edge *tail = tails->data;
-			len = calc_road_length(tail);
-			if (len > lengths[tail->owner])
-				lengths[tail->owner] = len;
-                        tails = g_list_remove(tails, tail);
-                }
+		len = find_longest_road_recursive (edge);
+		if (len > lengths[edge->owner]) lengths[edge->owner] = len;
 	}
-
 	return FALSE;
 }
 
 /* Zero the visited attribute for all edges and nodes.
  */
-static gboolean zero_visited(Map *map, Hex *hex, void *closure)
+static gboolean zero_visited(UNUSED(Map *map), Hex *hex, UNUSED(void *closure))
 {
 	gint idx;
 
 	for (idx = 0; idx < numElem(hex->edges); idx++) {
 		Edge *edge = hex->edges[idx];
-		edge->visited = 0;
+		edge->visited = FALSE;
 	}
 	for (idx = 0; idx < numElem(hex->nodes); idx++) {
 		Node *node = hex->nodes[idx];
-		node->visited = 0;
+		node->visited = FALSE;
 	}
 
 	return FALSE;
 }
 
-/* New improved algorithm for finding the longest road for each
- * player.  For each connected group of edges made of roads, ships,
- * and bridges, build a list of "tail" edges which are only connected
- * at one end.  If there are no such edges, choose one edge at random.
- * 
- * For each "tail" edge in the list, count the maximum length path
- * that can be traced from that edge.
- *
- * Mark the entire group of edges as searched.
- *
- * The method for finding connected edge groups is a simple map
- * traversal.  Whenever an edge is found which is not marked as
- * searched, it is part of a new group.
- *
- * Marking is done as follows;
- * 1 - Traverse map and zero the visited attribute of each edge / node.
- * 2 - Traverse map looking for edges with visited == 0, when one is
- *     found;
- * 2a - Do exhaustive search of all edges in group setting visited to
- *      1, build list of "tail" edges.  At the end of this process,
- *      all edges in group will have visited == 1.
- * 2b - From each tail edge, find the longest path that can be traced
- *      from that edge.  When visiting an edge / group, visited is set to
- *      2.  When unwinding the call stack, visited is restored to 1.
+/* Finding the longest road:
+ * 1 - set the visited attribute of all edges and nodes to FALSE
+ * 2 - for every edge, find the longest road using this one as a tail
  */
 void map_longest_road(Map *map, gint *lengths, gint num_players)
 {
@@ -930,7 +861,7 @@ void map_longest_road(Map *map, gint *lengths, gint num_players)
 
 /* Determine the maritime trading capabilities for the specified player
  */
-static gboolean find_maritime(Map *map, Hex *hex, MaritimeInfo *info)
+static gboolean find_maritime(UNUSED(Map *map), Hex *hex, MaritimeInfo *info)
 {
 	if (hex->terrain != SEA_TERRAIN || hex->resource == NO_RESOURCE)
 		return FALSE;
