@@ -1,12 +1,23 @@
-/*
- * Gnocatan: a fun game.
- * (C) 1999 the Free Software Foundation
+/* Gnocatan - Implementation of the excellent Settlers of Catan board game.
+ *   Go buy a copy.
  *
- * Author: Dave Cole.
+ * Copyright (C) 1999 the Free Software Foundation
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * Implementation of the excellent Settlers of Catan board game.  Go
- * buy a copy.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
 #ifndef __quoteinfo_h
 #define __quoteinfo_h
 
@@ -32,8 +43,10 @@ typedef struct {
 	GList *quotes;
 } QuoteList;
 
-QuoteList *quotelist_new(void);
-void quotelist_free(QuoteList *list);
+/** Create a new quote list, and remove the old list if needed */
+void quotelist_new(QuoteList **list);
+/** Free the QuoteList (if needed), and set it to NULL */
+void quotelist_free(QuoteList **list);
 QuoteInfo *quotelist_add_domestic(QuoteList *list, gint player_num,
 				  gint quote_num, gint *supply, gint *receive);
 QuoteInfo *quotelist_add_maritime(QuoteList *list,
