@@ -1,3 +1,25 @@
+/* Gnocatan - Implementation of the excellent Settlers of Catan board game.
+ *   Go buy a copy.
+ *
+ * Copyright (C) 1999 the Free Software Foundation
+ * Copyright (C) 2003 Bas Wijnen <b.wijnen@phys.rug.nl>
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -16,9 +38,11 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
+#include <sys/dir.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <time.h>
 
 #include <glib.h>
 
@@ -29,6 +53,12 @@ typedef enum {
 	META_SERVER
 } ClientType;
 
+typedef union {
+	struct sockaddr sa;
+	struct sockaddr_in in;
+	struct sockaddr_in6 in6;
+} sockaddr_t;
+
 typedef struct _Client Client;
 struct _Client {
 	ClientType type;
@@ -40,21 +70,26 @@ struct _Client {
 	int read_len;
 	GList *write_queue;
 	gboolean waiting_for_close;
+	gint protocol_major;
+	gint protocol_minor;
 
 	/* The rest of the structure is only used for META_SERVER clients
 	 */
 	gchar *host;
-	gint port;
+	gchar *port;
 	gchar *version;
 	gint max;
 	gint curr;
-	gchar *map;
-	gchar *comment;
+	gchar *terrain;
+	gchar *title;
+	gchar *vpoints;
+	gchar *sevenrule;
 
 	gboolean send_hello;
 };
 
 static gchar *redirect_location;
+static gchar *myhostname;
 
 static GList *client_list;
 
@@ -63,7 +98,7 @@ static fd_set write_fds;
 static int accept_fd;
 static gint max_fd;
 
-static void client_printf(Client *client, char *fmt, ...);
+static void client_printf(Client *client, const char *fmt, ...);
 
 #define MINUTE 60
 #define HOUR (60 * MINUTE)
@@ -73,10 +108,10 @@ static void client_printf(Client *client, char *fmt, ...);
 #define SERVER_TIMEOUT (48 * HOUR) /* delay allowed between server updates */
 #define HELLO_TIMEOUT (8 * MINUTE) /* delay allowed between server hello */
 
-#define LOG
+/* #define LOG */
 
 #ifdef LOG
-static void debug(gchar *fmt, ...)
+static void debug(const gchar *fmt, ...)
 {
 	static FILE *fp;
 	va_list ap;
@@ -110,7 +145,7 @@ static void debug(gchar *fmt, ...)
 }
 #endif
 
-static void find_new_max_fd()
+static void find_new_max_fd(void)
 {
 	GList *list;
 
@@ -125,6 +160,8 @@ static void find_new_max_fd()
 
 static void client_free(Client *client)
 {
+	if (client->type == META_SERVER)
+		syslog(LOG_INFO, "server on port %s unregistered", client->port);
 	g_free(client);
 }
 
@@ -153,10 +190,14 @@ static void client_close(Client *client)
 		g_free(client->host);
 	if (client->version != NULL)
 		g_free(client->version);
-	if (client->map != NULL)
-		g_free(client->map);
-	if (client->comment != NULL)
-		g_free(client->comment);
+	if (client->terrain != NULL)
+		g_free(client->terrain);
+	if (client->title != NULL)
+		g_free(client->title);
+	if (client->vpoints != NULL)
+		g_free(client->vpoints);
+	if (client->sevenrule != NULL)
+		g_free(client->sevenrule);
 }
 
 static void client_hello(Client *client)
@@ -228,7 +269,7 @@ static void client_do_write(Client *client)
 	set_client_event_at(client);
 }
 
-static void client_printf(Client *client, char *fmt, ...)
+static void client_printf(Client *client, const char *fmt, ...)
 {
 	gchar buff[10240];
 	va_list ap;
@@ -255,20 +296,224 @@ static void client_list_servers(Client *client)
 		client_printf(client,
 			      "server\n"
 			      "host=%s\n"
-			      "port=%d\n"
+			      "port=%s\n"
 			      "version=%s\n"
 			      "max=%d\n"
-			      "curr=%d\n"
-			      "map=%s\n"
-			      "comment=%s\n"
-			      "end\n",
+			      "curr=%d\n",
 			      scan->host, scan->port, scan->version,
-			      scan->max, scan->curr, scan->map,
-			      scan->comment);
+			      scan->max, scan->curr);
+		if (client->protocol_major == 0) {
+			client_printf(client,
+				      "map=%s\n"
+				      "comment=%s\n",
+				      scan->terrain,
+				      scan->title);
+		}
+		else if (client->protocol_major >= 1) {
+			client_printf(client,
+				      "vpoints=%s\n"
+				      "sevenrule=%s\n"
+				      "terrain=%s\n"
+				      "title=%s\n",
+				      scan->vpoints,
+				      scan->sevenrule,
+				      scan->terrain,
+				      scan->title);
+		}
+		client_printf(client, "end\n");
 	}
 }
 
-static gboolean check_str_info(gchar *line, gchar *prefix, gchar **data)
+static GList *load_game_desc(gchar *fname, GList *titles)
+{
+	FILE *fp;
+	gchar line[512], *title;
+
+	if ((fp = fopen(fname, "r")) == NULL) {
+		g_warning("could not open '%s'", fname);
+		return NULL;
+	}
+	while (fgets(line, sizeof(line), fp) != NULL) {
+		gint len = strlen(line);
+
+		if (len > 0 && line[len - 1] == '\n')
+			line[len - 1] = '\0';
+		if (strncmp(line, "title ", 6) == 0) {
+			title = line+6;
+			title += strspn(title, " \t");
+			titles = g_list_insert_sorted(titles, strdup(title),
+										  (GCompareFunc)strcmp);
+			break;
+		}
+	}
+	fclose(fp);
+	return titles;
+}
+
+static GList *load_game_types( const gchar *path )
+{
+	DIR *dir;
+	gchar *fname;
+	long fnamelen;
+	struct dirent *ent;
+	GList *titles = NULL;
+
+	if ((dir = opendir(path)) == NULL) {
+		return NULL;
+	}
+
+	fnamelen = pathconf(path,_PC_NAME_MAX);
+	fname = malloc(1 + fnamelen);
+	for (ent = readdir(dir); ent != NULL; ent = readdir(dir)) {
+		gint len = strlen(ent->d_name);
+
+		if (len < 6 || strcmp(ent->d_name + len - 5, ".game") != 0)
+			continue;
+		g_snprintf(fname, fnamelen, "%s/%s", path, ent->d_name);
+		titles = load_game_desc(fname, titles);
+	}
+
+	closedir(dir);
+	free(fname);
+	return titles;
+}
+
+static void client_list_types(Client *client)
+{
+	GList *list;
+	const gchar *gnocatan_dir = getenv( "GNOCATAN_DIR" );
+	if( !gnocatan_dir )
+		gnocatan_dir = GNOCATAN_DIR_DEFAULT;
+	
+	list = load_game_types(gnocatan_dir);
+
+	for (; list != NULL; list = g_list_next(list)) {
+		client_printf(client, "title=%s\n", list->data);
+	}
+	g_list_free(list);
+}
+
+static void client_create_new_server(Client *client, gchar *line)
+{
+	char *terrain, *numplayers, *points, *sevens_rule, *numai, *type;
+	char *newenv[2] = { NULL, NULL };
+	int pid, fd;
+	socklen_t yes=1;
+	struct sockaddr_in sa;
+	char port[20];
+	const char *console_server;
+
+	line += strspn(line, " \t");
+	terrain = line;
+	line += strspn(line, "0123456789");
+	if (line == terrain)
+		goto bad;
+	*line++ = 0;
+	line += strspn(line, " \t");
+	numplayers = line;
+	line += strspn(line, "0123456789");
+	if (line == numplayers)
+		goto bad;
+	*line++ = 0;
+	line += strspn(line, " \t");
+	points = line;
+	line += strspn(line, "0123456789");
+	if (line == points)
+		goto bad;
+	*line++ = 0;
+	line += strspn(line, " \t");
+	sevens_rule = line;
+	line += strspn(line, "0123456789");
+	if (line == points)
+		goto bad;
+	*line++ = 0;
+	line += strspn(line, " \t");
+	numai = line;
+	line += strspn(line, "0123456789");
+	if (line == points)
+		goto bad;
+	*line++ = 0;
+	line += strspn(line, " \t");
+	type = line;
+	line += strlen(line)-1;
+	while( line >= type && isspace(*line) )
+		*line-- = 0;
+	if (line < type)
+		goto bad;
+
+	if (!(console_server = getenv("GNOCATAN_SERVER_CONSOLE")))
+		console_server = GNOCATAN_SERVER_CONSOLE_PATH;
+	
+	fd = socket(AF_INET, SOCK_STREAM, 0);
+	if (fd < 0) {
+		client_printf(client, "Creating socket failed: %s\n",
+					  strerror(errno));
+		return;
+	}
+	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes)) < 0) {
+		client_printf(client, "Setting socket reuse failed: %s\n",
+					  strerror(errno));
+		return;
+	}
+	sa.sin_family = AF_INET;
+	sa.sin_port = 0;
+	sa.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+	if (bind(fd, (struct sockaddr *)&sa, sizeof(sa)) < 0) {
+		client_printf(client, "Binding socket failed: %s\n",
+					  strerror(errno));
+		return;
+	}
+	yes = sizeof(sa);
+	if (getsockname(fd, (struct sockaddr *)&sa, &yes) < 0) {
+		client_printf(client, "Getting socket address failed: %s\n",
+					  strerror(errno));
+		return;
+	}
+	sprintf(port, "%d", sa.sin_port);
+
+	newenv[0] = g_malloc(strlen("GNOCATAN_META_SERVER=")+strlen(myhostname)+1);
+	strcpy(newenv[0], "GNOCATAN_META_SERVER=");
+	strcat(newenv[0], myhostname);
+
+	if ((pid = fork()) == -1) {
+		client_printf(client, "fork failed\n");
+	}
+	else if (pid == 0) {
+		int i;
+		for( i = 0; i < 255; ++i ) close(i);
+		open("/dev/null",O_RDONLY);
+		open("/dev/null",O_WRONLY);
+		open("/dev/null",O_RDWR);
+		execle( console_server, console_server,
+				"-g", type,
+				"-P", numplayers,
+				"-v", points,
+				"-R", sevens_rule,
+				"-T", terrain,
+				"-p", port,
+				"-c", numai,
+				"-k", "1200",
+				"-x",
+				"-r",
+				NULL,
+				newenv );
+		syslog(LOG_ERR, "cannot exec %s: %m", console_server);
+		exit(2);
+	}
+	else {
+		close(fd);
+	}
+
+	client_printf(client, "host=%s\n", myhostname);
+	client_printf(client, "port=%s\n", port);
+	client_printf(client, "started\n");
+	syslog(LOG_INFO, "new server started on port %s", port);
+	return;
+  bad:
+	client_printf(client, "Badly formatted request\n");
+}
+
+static gboolean check_str_info(gchar *line, const gchar *prefix, gchar **data)
 {
 	gint len = strlen(prefix);
 
@@ -280,7 +525,7 @@ static gboolean check_str_info(gchar *line, gchar *prefix, gchar **data)
 	return TRUE;
 }
 
-static gboolean check_int_info(gchar *line, gchar *prefix, gint *data)
+static gboolean check_int_info(gchar *line, const gchar *prefix, gint *data)
 {
 	gint len = strlen(prefix);
 
@@ -292,73 +537,126 @@ static gboolean check_int_info(gchar *line, gchar *prefix, gint *data)
 
 static void try_make_server_complete(Client *client)
 {
-	if (client->type == META_SERVER)
+	int ok = 0;
+	
+	if (client->type == META_SERVER) {
+		static int prev_curr = -1;
+		if (client->curr != prev_curr) {
+			syslog(LOG_INFO, "server on port %s: now %d players",
+				   client->port, client->curr);
+			prev_curr = client->curr;
+		}
 		return;
+	}
 
 	if (client->host != NULL
-	    && client->port > 0
+	    /* FIXME: client->port is a gchar *, because it should be possible
+	     * to give the name of a service instead of a port.  atoi obviously
+	     * doesn't allow this */
+	    && atoi (client->port) > 0
 	    && client->version != NULL
 	    && client->max >= 0
 	    && client->curr >= 0
-	    && client->map != NULL
-	    && client->comment != NULL) {
-		client->type = META_SERVER;
-		client->send_hello = strcmp(client->version, "0.3.0") >= 0;
+	    && client->terrain != NULL
+	    && client->title != NULL) {
+	    if (client->protocol_major < 1) {
+		if (!client->vpoints)
+		    client->vpoints = g_strdup("?");
+		if (!client->sevenrule)
+		    client->sevenrule = g_strdup("?");
+		ok = 1;
+	    }
+	    else {
+		if (client->vpoints != NULL
+		    && client->sevenrule != NULL)
+		    ok = 1;
+	    }
+	}
+	    
+	if (ok) {
+	    client->type = META_SERVER;
+	    client->send_hello = strcmp(client->version, "0.3.0") >= 0;
 	}
 }
 
-static gchar *get_peer_name(gint fd)
+static void get_peer_name(gint fd, gchar **hostname, gchar **servname)
 {
-	struct sockaddr_in peer;
+	sockaddr_t peer;
 	socklen_t peer_len;
 
 	peer_len = sizeof(peer);
-	if (getpeername(fd, &peer, &peer_len) < 0)
+	if (getpeername(fd, &peer.sa, &peer_len) < 0)
 		syslog(LOG_ERR, "getting peer name: %m");
 	else {
-		struct hostent *host_ent;
+		int err;
+		char host[NI_MAXHOST];
+		char port[NI_MAXSERV];
 
-		host_ent = gethostbyaddr((char*)&peer.sin_addr,
-					 sizeof(peer.sin_addr), AF_INET);
-		if (host_ent == NULL)
-			syslog(LOG_ERR, "resolving address: %s",
-			       hstrerror(h_errno));
-		else
-			return g_strdup(host_ent->h_name);
+		if((err = getnameinfo(&peer.sa, peer_len, host, NI_MAXHOST, port, NI_MAXSERV, 0)))
+			syslog(LOG_ERR, "resolving address: %s", gai_strerror(err));
+		else {
+			*hostname = g_strdup(host);
+			*servname = g_strdup(port);
+			return;
+		}
 	}
 
-	return g_strdup(inet_ntoa(peer.sin_addr));
+	*hostname = g_strdup("unknown");
+	*servname = g_strdup("unknown");
+	return;
 }
 
 static void client_process_line(Client *client, gchar *line)
 {
 	switch (client->type) {
 	case META_UNKNOWN:
-		/* All we want the client to do is identify themselves
-		 */
-		if (strcmp(line, "client") == 0) {
+	case META_CLIENT:
+		if (strncmp(line, "version ", 8) == 0) {
+			char *p = line+8;
+			client->protocol_major = atoi(p);
+			p += strspn(p, "0123456789");
+			if (*p == '.')
+				client->protocol_minor = atoi(p+1);
+		}
+		else if (strcmp(line, "listservers") == 0 ||
+			/* still accept "client" request from proto 0 clients
+			 * so we don't have to distinguish between client versions */
+			strcmp(line, "client") == 0) {
 			client->type = META_CLIENT;
 			client_list_servers(client);
 			client->waiting_for_close = TRUE;
+		} else if (strcmp(line, "listtypes") == 0) {
+			client->type = META_CLIENT;
+			client_list_types(client);
+			client->waiting_for_close = TRUE;
+		} else if (strncmp(line, "create ", 7) == 0) {
+			client->type = META_CLIENT;
+			client_create_new_server(client,line+7);
+			client->waiting_for_close = TRUE;
 		} else if (strcmp(line, "server") == 0) {
 			client->type = META_SERVER_ALMOST;
-			client->port = client->max = client->curr = -1;
-			client->host = get_peer_name(client->fd);
+			client->max = client->curr = -1;
+			get_peer_name(client->fd, &client->host, &client->port);
+		}
+		else {
+			client_printf(client, "bad command\n");
+			client->waiting_for_close = TRUE;
 		}
-		break;
-	case META_CLIENT:
-		/* Clients are not allowed to do anything but receive
-		 * a server list
-		 */
 		break;
 	case META_SERVER:
 	case META_SERVER_ALMOST:
-		if (check_int_info(line, "port=", &client->port)
+		if (check_str_info(line, "host=", &client->host)
+		    || check_str_info(line, "port=", &client->port)
 		    || check_str_info(line, "version=", &client->version)
 		    || check_int_info(line, "max=", &client->max)
 		    || check_int_info(line, "curr=", &client->curr)
-		    || check_str_info(line, "map=", &client->map)
-		    || check_str_info(line, "comment=", &client->comment))
+		    || check_str_info(line, "terrain=", &client->terrain)
+		    || check_str_info(line, "title=", &client->title)
+		    || check_str_info(line, "vpoints=", &client->vpoints)
+		    || check_str_info(line, "sevenrule=", &client->sevenrule)
+		    /* meta-protocol 0.0 compat */
+		    || check_str_info(line, "map=", &client->terrain)
+		    || check_str_info(line, "comment=", &client->title))
 			try_make_server_complete(client);
 		else if (strcmp(line, "begin") == 0)
 			client_close(client);
@@ -444,15 +742,15 @@ static void client_do_read(Client *client)
 	set_client_event_at(client);
 }
 
-static void accept_new_client()
+static void accept_new_client(void)
 {
 	int fd;
-	struct sockaddr_in addr;
+	sockaddr_t addr;
 	socklen_t addr_len;
 	Client *client;
 
 	addr_len = sizeof(addr);
-	fd = accept(accept_fd, &addr, &addr_len);
+	fd = accept(accept_fd, &addr.sa, &addr_len);
 	if (fd < 0) {
 		syslog(LOG_ERR, "accepting connection: %m");
 		return;
@@ -464,19 +762,22 @@ static void accept_new_client()
 	client_list = g_list_append(client_list, client);
 
 	client->type = META_UNKNOWN;
+	client->protocol_major = 0;
+	client->protocol_minor = 0;
 	client->fd = fd;
 
 	if (redirect_location != NULL) {
 		client_printf(client, "goto %s\n", redirect_location);
 		client->waiting_for_close = TRUE;
 	} else {
-		client_printf(client, "welcome to the gnocatan-meta-server\n");
+		client_printf(client, "welcome to the gnocatan-meta-server version %s\n",
+					  META_PROTOCOL_VERSION);
 		FD_SET(client->fd, &read_fds);
 	}
 	set_client_event_at(client);
 }
 
-static struct timeval *find_next_delay()
+static struct timeval *find_next_delay(void)
 {
 	GList *list;
 	static struct timeval timeout;
@@ -501,7 +802,15 @@ static struct timeval *find_next_delay()
 	return &timeout;
 }
 
-static void check_timeouts()
+static void reap_children(void)
+{
+	int dummy;
+	
+	while( waitpid(-1, &dummy, WNOHANG) > 0 )
+		;
+}
+
+static void check_timeouts(void)
 {
 	time_t now = time(NULL);
 	GList *list;
@@ -520,7 +829,7 @@ static void check_timeouts()
 	}
 }
 
-static void select_loop()
+static void select_loop(void)
 {
 	for (;;) {
 		fd_set read_res;
@@ -529,6 +838,7 @@ static void select_loop()
 		struct timeval *timeout;
 		GList *list;
 
+		reap_children();
 		check_timeouts();
 		timeout = find_next_delay();
 
@@ -576,39 +886,61 @@ static void select_loop()
 	}
 }
 
-static gboolean setup_accept_sock(gint port)
+static gboolean setup_accept_sock(const gchar *port)
 {
-	struct sockaddr_in addr;
-	int yes;
+	int err;
+	struct addrinfo hints, *ai, *aip;
+	int fd, yes;
 
-	memset(&addr, 0, sizeof(addr));
+	memset(&hints, 0, sizeof(hints));
 
-	addr.sin_family = AF_INET;
-	addr.sin_addr.s_addr = INADDR_ANY;
-	addr.sin_port = htons(port);
+	hints.ai_family = PF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_protocol = IPPROTO_TCP;
+	hints.ai_flags = AI_PASSIVE;
 
-	max_fd = accept_fd = socket(AF_INET, SOCK_STREAM, 0);
-	if (accept_fd < 0) {
-		syslog(LOG_ERR, "creating socket: %m");
+	if((err = getaddrinfo(NULL, port, &hints, &ai)) || !ai) {
+		syslog(LOG_ERR, "creating struct addrinfo: %s", gai_strerror(errno));
 		return FALSE;
 	}
-	if (bind(accept_fd, &addr, sizeof(addr)) < 0) {
-		syslog(LOG_ERR, "binding socket: %m");
+
+	for(aip = ai; aip; aip = aip->ai_next) {
+		fd = socket(aip->ai_family, SOCK_STREAM, 0);
+		if (fd < 0) {
+			continue;
+		}
+		yes = 1;
+		if (setsockopt(fd,
+					   SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes)) < 0) {
+			close(fd);
+			continue;
+		}
+		if (bind(fd, aip->ai_addr, aip->ai_addrlen) < 0) {
+			close(fd);
+			continue;
+		}
+
+		break;
+	}
+	
+	if (!aip) {
+		syslog(LOG_ERR, "error creating listening socket: %m\n");
+		freeaddrinfo(ai);
 		return FALSE;
 	}
-	yes = 1;
-	if (setsockopt(accept_fd,
-		       SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes)) < 0) {
-		syslog(LOG_ERR, "setting socket address reuse: %m");
-		return FALSE;
-	}
-	if (fcntl(accept_fd, F_SETFL, O_NDELAY) < 0) {
+
+	accept_fd = max_fd = fd;
+	freeaddrinfo(ai);
+	
+	if (fcntl(fd, F_SETFL, O_NDELAY) < 0) {
 		syslog(LOG_ERR, "setting socket non-blocking: %m");
+		close(accept_fd);
 		return FALSE;
 	}
 
-	if (listen(accept_fd, 5) < 0) {
+	if (listen(fd, 5) < 0) {
 		syslog(LOG_ERR, "during listen on socket: %m");
+		close(accept_fd);
 		return FALSE;
 	}
 
@@ -616,7 +948,7 @@ static gboolean setup_accept_sock(gint port)
 	return TRUE;
 }
 
-static void convert_to_daemon()
+static void convert_to_daemon(void)
 {
 	pid_t pid;
 
@@ -644,6 +976,23 @@ static void convert_to_daemon()
 	umask(0);
 }
 
+static void setmyhostname( void )
+{
+	char hbuf[256];
+	struct hostent *hp;
+
+	myhostname = NULL;
+	if (gethostname(hbuf, sizeof(hbuf))) {
+		perror("gethostname");
+		return;
+	}
+	if (!(hp = gethostbyname(hbuf))) {
+		herror("gnocatan-meta-server");
+		return;
+	}
+	myhostname = strdup(hp->h_name);
+}
+
 int main(int argc, char *argv[])
 {
 	int c;
@@ -663,9 +1012,11 @@ int main(int argc, char *argv[])
 	if (make_daemon)
 		convert_to_daemon();
 
-	if (!setup_accept_sock(5557))
+	setmyhostname();
+	if (!setup_accept_sock(GNOCATAN_DEFAULT_META_PORT))
 		return 1;
 
+	syslog(LOG_INFO, "Gnocatan meta server started.");
 	select_loop();
 
 	return 0;
